<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "abstractcomponent.class.php" );
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "functions.php" );

abstract class AbstractAddress extends AbstractComponent {
  public $addressid;
  public $address1;
  public $address2;
  public $address3;
  public $city;
  public $state;
  public $postalcode;
  public $countrycode;

  public $changed;
  public $original;
  public $context;
  public $fields = null;

  public function abstract_address() {
  }

  // SQL to read one record from database
  abstract public function read_sql($id);
  // SQL to read all records from database
  abstract public function read_all_sql($id);
  // SQL to update one record in database
  abstract public function update_sql($setstring);
  // SQL to insert one record into the database
  abstract public function insert_sql($columns, $values );
  // return the internally used id (e.g., $this->personid)
  abstract public function inner_id();
  // load the specific extra fields (such as PersonID and Phone)
  abstract public function inner_load( $row, $context );
  // the extra fields
  abstract public function additional_fields();
  // save the specific extra fields (such as Phone)
  // also send necessary notification emails
  abstract public function inner_save();
  // return a pretty print string of the extra fields; parameter is the pretty-print so far
  abstract public function inner_pretty_print_ascii($string, $lineterminator);

  public function load( $row, $context ) {
  	$this->inner_load( $row, $context );
    $this->addressid = isset($row['AddressID']) ? stripslashes($row['AddressID']) : '';
    $this->address1 = isset($row['Address1']) ? stripslashes($row['Address1']) : '';
    $this->address2 = isset($row['Address2']) ? stripslashes($row['Address2']) : '';
    $this->address3 = isset($row['Address3']) ? stripslashes($row['Address3']) : '';
    $this->city = isset($row['City']) ? stripslashes($row['City']) : '';
    $this->state = isset($row['ProvStateRegion']) ? stripslashes($row['ProvStateRegion']) : '';
    $this->postalcode = isset($row['PostalCode']) ? stripslashes($row['PostalCode']) : '';
    $this->countrycode = isset($row['CCode']) ? stripslashes($row['CCode']) : '';

    $this->changed = null;
    $this->original = null;
    $this->context = null;
    $y = $this->fields;
    $this->fields = null;
    $x = serialize($this);
	$this->changed = array();
	$this->original = $x;
    $this->context = $context;
    $this->fields = $y;
    if( $this->fields == null ) {
    	$this->initialize_fields_variable();
    }
  }
  public function initialize_fields_variable() {
	$result = foundation_sql( "SELECT CCode, en_Description FROM SYS_Countries ORDER BY en_Description");
	$options = array();
	while( $row = mysql_fetch_assoc($result) ) {
		$s = $row['en_Description'];
		if( strlen($s) > 23 ) {
			$s = substr($s, 0, 20) . '...';
		}
		$options[$row['CCode']] = $s;
	}
	$this->fields = array_merge( array (
      array( 'key' => 'address1', 'label'=>'Address 1', 'check' => '200'),
      array( 'key' => 'address2', 'label'=>'Address 2', 'check' => '200'),
      array( 'key' => 'address3', 'label'=>'Address 3', 'check' => '200'),
      array ('key'=>'city', 'label'=>'City', 'check' => 'req 100'),
      array ('key'=>'state', 'label'=>'State/Province', 'check' => '100'),
      array ('key'=>'postalcode', 'label'=>'Postal Code', 'check' => '100'),
      array ('key'=>'countrycode', 'label'=>'Country', 'type'=>'select', 'options'=>$options, 'check'=>'req 50' ) ),
      $this->additional_fields()
	);
  }
  /* Load from the database */
  public function read( $id, $context ) {
   $this->context = $context;
   if(strlen($id) < 1) {
        echo "Error: ID was blank";
   }
   if( preg_match( '/(.*)\/\*/', $id, $matches )) {
		$result = foundation_sql( $this->read_first_sql($matches[1]));
   } else {
		$result = foundation_sql( $this->read_sql($id));
   }
   $row = mysql_fetch_assoc($result);
   $this->load( $row, $this->context );
  }
  /*
   * Post-Action: none
   */
  public function save() {
 	if($this->addressid == "") {
		// if there is no saved address yet, do an insert..
		$values = "";
		$fields = "";
	    foreach($this->changed as $x){
			if($x=='address1'){
			    $values.="'".addslashes($this->address1)."'".",";
			    $fields.="Address1,";
			}
			if($x=='address2'){
			    $values.="'".addslashes($this->address2)."'".",";
			    $fields.="Address2,";
			}
	        if($x=='address3'){
			    $values.="'".addslashes($this->address3)."'".",";
			    $fields.="Address3,";
			}
	        if($x=='city'){
			    $values.="'".addslashes($this->city)."'".",";
			    $fields.="City,";
			}
	        if($x=='state'){
			    $values.="'".addslashes($this->state)."'".",";
			    $fields.="ProvStateRegion,";
			}
	        if($x=='postalcode'){
			    $values.="'".addslashes($this->postalcode)."'".",";
			    $fields.="PostalCode,";
			}
	        if($x=='countrycode'){
			    $values.="(SELECT CCode FROM SYS_Countries WHERE CCode = '".addslashes($this->countrycode)."'),";
			    $fields.="CCode,";
			}
	    }
	    if( $fields != '' ) {
			// Cut the last comma
			$fields = substr($fields, 0, strrpos($fields, ","));
			$values = substr($values, 0, strrpos($values, ","));
			$sql = $this->insert_sql( $fields, $values );
	 		$result = foundation_sql($sql);
			$dbh = $this->context->database( 'foundation' );
	    }
	} else {
		// if there is already an address row, then do an update..
	    $setstring = "";
	    foreach($this->changed as $x){
			if($x=='address1'){$setstring.=",Address1='".addslashes($this->address1)."'";}
			if($x=='address2'){$setstring.=",Address2='".addslashes($this->address2)."'";}
			if($x=='address3'){$setstring.=",Address3='".addslashes($this->address3)."'";}
			if($x=='city'){$setstring.=",City='".addslashes($this->city)."'";}
			if($x=='state'){$setstring.=",ProvStateRegion='".addslashes($this->state)."'";}
			if($x=='postalcode'){$setstring.=",PostalCode='".addslashes($this->postalcode)."'";}
			if($x=='countrycode'){$setstring.=",CCode=(SELECT CCode FROM SYS_Countries WHERE CCode = '".addslashes($this->countrycode)."')";}
	    }
	    if($setstring!=""){
			$setstring=substr($setstring, 1);
			$sql= $this->update_sql( $setstring );
			$result=foundation_sql( $sql );
        }
	}
    $this->inner_save();
	/* if there was a database problem, my (the object's) internal state may differ from
	 * the database state. We solve this by re-reading the database state into the object. */
  	$this->read($this->inner_id(), $this->context );
  }

  public function inner_html() {
      $this->html_show_fields($this->fields, 'edit');
  }

  public function save_action() {
    $errors = $this->html_save_fields($this->fields);
    if ($errors) {
      $this->edit_html();
    } else {
      $this->save();
      $this->inner_html();
    }
  }

  public function pretty_print_ascii( $lineterminator = "\n" ) {
	$rtrn = "";
	if( $this->address1 ) $rtrn .= $this->address1 . $lineterminator;
	if( $this->address2 ) $rtrn .= $this->address2 . $lineterminator;
	if( $this->address3 ) $rtrn .= $this->address3 . $lineterminator;
	if( $this->city ) $rtrn .= $this->city . $lineterminator;
	if( $this->state ) $rtrn .= $this->state . $lineterminator;
	if( $this->postalcode ) $rtrn .= $this->postalcode . $lineterminator;
	if( $this->countrycode ) {
		$result = foundation_sql( "SELECT * FROM SYS_Countries WHERE CCode = '$this->countrycode'");
		$row = mysql_fetch_assoc($result);
		$rtrn .= $row['en_Description'] . $lineterminator;
	}
	$rtrn = $this->inner_pretty_print_ascii($rtrn, $lineterminator);
	return $rtrn;
  }

  public function changes_print_ascii( $changes, $original, $lineterminator = "\n" ) {
  	$rtrn = "";
  	foreach( $changes as $change ) {
  		foreach( $this->fields as $rec ) {
  			if( $rec['key'] == $change &&
  				isset($rec['transient']) && 
  				$rec['transient'] == true)
  			continue 2;
  		}
		$newvalue = "\$newvalue = \$this->$change;";
		eval($newvalue);
		$oldvalue = "\$oldvalue = \$original->$change;";
		eval($oldvalue);
		$rtrn .= "  $change : $oldvalue => $newvalue\n";
    }
    return $rtrn;
  }
}

?>
