<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
class Context {
  var $hash;
  public $databases;
  public $max_affected_rows = 100;  // used in tests, set to 1 after each advance and press
  var $showerrors  = false; // true if we should display errors to the end user
  var $testmode    = false; // true if running a test, false otherwise
  var $devmode     = false; // true if development, false if production
  var $initmode    = false; // if checking, modifying, creating, and initializing (CMCI) tables then

  	/* '' means CMCI the normal tables
  	 * '_prototype' means CMCI the prototype tables
  	 * false means ..CI in-memory tables populated from prototype tables
  	 */
  /*
   *             +---------- devmode ----------------+
   *             |     TRUE      |     FALSE         |
   * +-testmode/-+               |                   |
   * +-initmode--+---------------+-------------------+---
   * |           | running tests | running tests     | saves database
   * |   TRUE /  | on developer  | on staging or     | for future
   * |   TRUE    | workstation   | production        | investigation
   * |           |               | server            |
   * +-----------+---------------+-------------------+---
   * |           | exploring     | exploring         | uses engine=memory
   * |   TRUE /  | on developer  | on staging or     | tables for optimal
   * |   FALSE   | workstation   | production        | server performance
   * |           |               | server            |
   * +-----------+---------------+-------------------+---
   * |           | running demo  | running real      |
   * |  FALSE    | on developer  | portal on staging | uses real databases
   * |           | workstation   | or production     |
   * |           |               | server            |
   * +-----------+---------------+-------------------+---
   */
  var $trace_buffer;

  /*
   * =========== Constructor ==============
   */
   public function Context() {
	 $this->hash = array();
	 $this->databases = array();
   }
  /*
   * =========== Generic functions ==============
   */
  public function get( $key ) {
    return $this->hash[$key];
  }

  public function set( $key, $value ) {
    $this->hash[$key] = $value;
  }
  /*
   * ========== Specialized functions ==============
   */
  /*
   * effective user (usually the logged in user)
   */
  public function user() {
    if (isset($this->hash['USER'])) {
      return $this->get("USER");
    } else {
      return "nobody";
    }
  }

  // Return the email address if we know it.  Otherwise return false so we can detect it easily.
  public function user_email() {
  	if(isset($this->hash['USER_EMAIL'])) {
  		return $this->get('USER_EMAIL');
  	} else {
  		return false;
  	}
  }
  public function user_email_name() {
  	if(isset($this->hash['USER_ENAME'])) {
  		return $this->get('USER_ENAME');
  	} else {
  		return false;
  	}
  }

  // invoked by external_entry_point to store email address on login.
  public function store_user_email() {
  	$user = $this->user();
  	if($user == 'nobody') {
  		return;
  	}
  	$result = foundation_sql("SELECT FName, LName, EMail FROM People WHERE PersonID = '$user'");
	$row = mysql_fetch_assoc($result);
	if(isset($row['EMail'])) {
		$this->set('USER_EMAIL', $row['EMail']);
		$this->set('USER_ENAME', $row['FName'] . ' ' . $row['LName']);
	}
  }

  /*
   * The SQL function to return the current time
   */
  public function now() {
  	return $this->get("NOW");
  }
  /*
   * Record a database record
   */
  public function setdatabase( $key, $host, $user, $pwd, $db ) {
    $rec = array() ;
    $rec['HOST'] = $host;
    $rec['USERNAME'] = $user;
    $rec['PASSWORD'] = $pwd;
    $rec['DATABASE'] = $db;
    $rec['CONNECTION'] = null;
    $this->databases[$key] = $rec;
  }
  /*
   * Open a database
   * Perhaps in the future connection can be cached and shared
   */
  public function database( $key ) {
    if($this->testmode) {
		$rec = $this->databases[$key . '_test'];
	} else {
		$rec = $this->databases[$key];
	}
    $dbh = $rec['CONNECTION'];
    if( $dbh == null ) {
      $dbh = mysql_connect( $rec['HOST'], $rec['USERNAME'], $rec['PASSWORD']); //TODO add the ,true back again after figuring out the PHP-MySQL interaction
      $this->databases[$key]['CONNECTION'] = $dbh;
    }
    $this->set('DBHANDLEMAP ' . $dbh, $key);
    mysql_select_db( $rec['DATABASE'], $dbh );
    return $dbh;
  }

  public function database_name( $key ) {
    if($this->testmode) {
		$rec = $this->databases[$key . '_test'];
	} else {
		$rec = $this->databases[$key];
	}
    return $rec['DATABASE'];
  }
  public function database_name_for_handle($dbh) {
  	return $this->get('DBHANDLEMAP ' . $dbh);
  }
  /*
   * ============ Functions of some kind ============
   */
  public function trace ( $string ) {
    if ( $this->testmode ) {
      $this->trace_buffer .= $string;
      $this->trace_buffer .= "<br>\n";
    }
  }

  public function getActionLink( $action, $obj, $param = null ) {
    if( $this->testmode ) {
      return $obj->get_explore_url_for_action_named($action);
    } else {
      $param = ($param) ? "&param=$param" : '';
      return "javascript:portal_get('" . $obj->id() . "','" . $action . $param . "');";
    }
  }

 public function getSubmitLink( $action, $obj, $formid = null ) {
   if ( $this->testmode ) {
     return "javascript:alert('Not active in test');return false;";
   } else {
   	 $formstr = ($formid == null) ? 'this' : "document.getElementById('$formid')";
     return "javascript:portal_submit('" . $obj->id() . "','" . $action . "',$formstr);return false;";
   }
 }
}

?>