<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "abstractschemachecker.class.php" );
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "functions.php" );
/*
 * For a complete, ready-to-copy, example of a component's schema checker,
 * please see the committer_vote component.
 */

class system_SchemaChecker extends AbstractSchemaChecker {
  public function check_and_modify( $context ) {
	return  $this->check_immediate_emails( $context )
		 && $this->check_daily_emails( $context )
		 && $this->check_modlog( $context )
		 && $this->check_profiles( $context );
  }

  public function check_immediate_emails( $context ) {
		$schemas = array();
$schemas[1] = "
+-----------+----------+------+-----+---------+----------------+
| Field     | Type     | Null | Key | Default | Extra          |
+-----------+----------+------+-----+---------+----------------+
| id        | int(11)  | NO   | PRI | NULL    | auto_increment |
| component | text     | YES  |     | NULL    |                |
| class     | text     | YES  |     | NULL    |                |
| param1    | text     | YES  |     | NULL    |                |
| param2    | text     | YES  |     | NULL    |                |
| param3    | text     | YES  |     | NULL    |                |
| param4    | text     | YES  |     | NULL    |                |
| sent      | datetime | YES  |     | NULL    |                |
+-----------+----------+------+-----+---------+----------------+
";
$schemas[2] = "
+-----------+---------------+------+-----+----------------------------+----------------+
| Field     | Type          | Null | Key | Default                    | Extra          |
+-----------+---------------+------+-----+----------------------------+----------------+
| id        | int(11)       | NO   | PRI | NULL                       | auto_increment |
| component | text          | YES  |     | NULL                       |                |
| class     | text          | YES  |     | NULL                       |                |
| fromaddr  | varchar(1024) | NO   |     | portal-noreply@eclipse.org |                |
| param1    | text          | YES  |     | NULL                       |                |
| param2    | text          | YES  |     | NULL                       |                |
| param3    | text          | YES  |     | NULL                       |                |
| param4    | text          | YES  |     | NULL                       |                |
| sent      | datetime      | YES  |     | NULL                       |                |
+-----------+---------------+------+-----+----------------------------+----------------+
";
	return $this->check_and_modify_table( 'myfoundation', 'immediate_emails', $schemas, $context );
  }
  public function create_immediate_emails($suffix ) {
	return "CREATE TABLE immediate_emails$suffix ( id INT AUTO_INCREMENT PRIMARY KEY, component
    	     TEXT, class TEXT, fromaddr varchar(1024) NOT NULL DEFAULT 'portal-noreply@eclipse.org',
		param1 TEXT, param2 TEXT, param3 TEXT, param4 TEXT, sent DATETIME )";
  }

  public function modify_immediate_emails_1_2( $suffix, $dbh ) {
        echo "adding fromaddr column <br>\n";
    portal_sql( "alter table immediate_emails$suffix add column fromaddr varchar(1024) NOT NULL DEFAULT 'portal-noreply@eclipse.org' after class;");
        echo "setting existing fromaddr fields to portal-noreply@eclipse.org <br>\n";
    portal_sql( "UPDATE immediate_emails$suffix SET fromaddr = 'portal-noreply@eclipse.org'
		WHERE fromaddr IS NULL");
  }

  public function check_daily_emails( $context ) {
		$schemas = array();
$schemas[1] = "
+-----------+----------+------+-----+---------+----------------+
| Field     | Type     | Null | Key | Default | Extra          |
+-----------+----------+------+-----+---------+----------------+
| id        | int(11)  | NO   | PRI | NULL    | auto_increment |
| component | text     | YES  |     | NULL    |                |
| class     | text     | YES  |     | NULL    |                |
| param1    | text     | YES  |     | NULL    |                |
| param2    | text     | YES  |     | NULL    |                |
| param3    | text     | YES  |     | NULL    |                |
| param4    | text     | YES  |     | NULL    |                |
| sent      | datetime | YES  |     | NULL    |                |
+-----------+----------+------+-----+---------+----------------+
";
$schemas[2] = "
+-----------+---------------+------+-----+----------------------------+----------------+
| Field     | Type          | Null | Key | Default                    | Extra          |
+-----------+---------------+------+-----+----------------------------+----------------+
| id        | int(11)       | NO   | PRI | NULL                       | auto_increment |
| component | text          | YES  |     | NULL                       |                |
| class     | text          | YES  |     | NULL                       |                |
| fromaddr  | varchar(1024) | NO   |     | portal-noreply@eclipse.org |                |
| param1    | text          | YES  |     | NULL                       |                |
| param2    | text          | YES  |     | NULL                       |                |
| param3    | text          | YES  |     | NULL                       |                |
| param4    | text          | YES  |     | NULL                       |                |
| sent      | datetime      | YES  |     | NULL                       |                |
+-----------+---------------+------+-----+----------------------------+----------------+
";

	return $this->check_and_modify_table( 'myfoundation', 'daily_emails', $schemas, $context );
  }
  public function create_daily_emails( $suffix) {
    return "CREATE TABLE daily_emails$suffix ( id INT AUTO_INCREMENT PRIMARY KEY, component
             TEXT, class TEXT, fromaddr varchar(1024) NOT NULL DEFAULT 'portal-noreply@eclipse.org',
		param1 TEXT, param2 TEXT, param3 TEXT, param4 TEXT, sent DATETIME )";
  }

  public function modify_daily_emails_1_2( $suffix, $dbh ) {
        echo "adding fromaddr column <br>\n";
    portal_sql( "alter table daily_emails$suffix add column fromaddr varchar(1024) NOT NULL DEFAULT 'portal-noreply@eclipse.org' after class;");
        echo "setting existing fromaddr fields to portal-noreply@eclipse.org <br>\n";
    portal_sql( "UPDATE daily_emails$suffix SET fromaddr = 'portal-noreply@eclipse.org' WHERE fromaddr IS NULL");
  }

  public function check_modlog( $context ) {
		$schemas = array();
$schemas[1] = "
+-------------+------------------+------+-----+---------------------+----------------+
| Field       | Type             | Null | Key | Default             | Extra          |
+-------------+------------------+------+-----+---------------------+----------------+
| LogID       | int(10) unsigned | NO   | PRI | NULL                | auto_increment |
| LogTable    | varchar(100)     | NO   |     | NULL                |                |
| PK1         | varchar(100)     | NO   |     | NULL                |                |
| PK2         | varchar(100)     | NO   |     | NULL                |                |
| LogAction   | varchar(255)     | NO   |     | NULL                |                |
| PersonID    | varchar(20)      | NO   |     | NULL                |                |
| ModDateTime | datetime         | NO   |     | 0000-00-00 00:00:00 |                |
+-------------+------------------+------+-----+---------------------+----------------+
";
	return $this->check_and_modify_table( 'foundation', 'SYS_ModLog', $schemas, $context );
  }
  public function create_SYS_ModLog($suffix ) {
	return "CREATE TABLE `SYS_ModLog$suffix` ( `LogID` int(10) unsigned NOT NULL auto_increment,
			`LogTable` varchar(100) NOT NULL default '',
			`PK1` varchar(100) NOT NULL default '',
			`PK2` varchar(100) NOT NULL default '',
			`LogAction` varchar(255) NOT NULL default '',
			`PersonID` varchar(20) NOT NULL default '',
			`ModDateTime` datetime NOT NULL default '0000-00-00 00:00:00',
			PRIMARY KEY (`LogID`) ) DEFAULT CHARSET=latin1";
  }

  // Profiles for users from the Bugzilla database
  public function check_profiles( $context ) {
  	$schemas = array();
  	$schemas[1] = "
+----------------+--------------+------+-----+---------------------+----------------+
| Field          | Type         | Null | Key | Default             | Extra          |
+----------------+--------------+------+-----+---------------------+----------------+
| userid         | mediumint(9) | NO   | PRI | NULL                | auto_increment |
| login_name     | varchar(255) | NO   | UNI |                     |                |
| cryptpassword  | varchar(128) | YES  |     | NULL                |                |
| realname       | varchar(255) | YES  |     | NULL                |                |
| disabledtext   | mediumtext   | NO   |     |                     |                |
| mybugslink     | tinyint(4)   | NO   |     | 1                   |                |
| refreshed_when | datetime     | NO   |     | 0000-00-00 00:00:00 |                |
| extern_id      | varchar(64)  | YES  |     | NULL                |                |
+----------------+--------------+------+-----+---------------------+----------------+
";
  	return $this->check_and_modify_table( 'bugzilla', 'profiles', $schemas, $context );
  }
  public function create_profiles($suffix) {
  	return "CREATE TABLE `profiles$suffix` (
			`userid` mediumint(9) NOT NULL auto_increment,
			`login_name` varchar(255) NOT NULL default '',
			`cryptpassword` varchar(128) default NULL,
			`realname` varchar(255) default NULL,
			`disabledtext` mediumtext NOT NULL,
			`mybugslink` tinyint(4) NOT NULL default '1',
			`refreshed_when` datetime NOT NULL default '0000-00-00 00:00:00',
			`extern_id` varchar(64) default NULL,
			PRIMARY KEY  (`userid`),
			UNIQUE KEY `profiles_login_name_idx` (`login_name`) )";
  }

  }
?>
